@@ -496,7 +496,7 @@ void *readFAT(struct sFileSystem *fs, uint16_t nr) {
 */
 
 	assert(fs != NULL);
-	assert(nr < fs->bs.xxFATxx.FAT12_16_32.BS_NumFATs);
+	assert(nr < fs->FATCount);
 
 	uint32_t FATSizeInBytes;
 	off_t BSOffset;
@@ -509,13 +509,27 @@ void *readFAT(struct sFileSystem *fs, uint16_t nr) {
 		stderror();
 		return NULL;
 	}
-	BSOffset = (off_t)SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
+	
+	switch(fs->FATType) {
+	case FATTYPE_FAT32:
+	case FATTYPE_FAT16:
+	case FATTYPE_FAT12:
+	  BSOffset = (off_t)SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
+	  break;
+	case FATTYPE_EXFAT:
+	  BSOffset = (off_t)SwapInt32(fs->bs.xxFATxx.exFAT.fat_sector_start) * fs->sectorSize;
+	  break;
+	default:
+	  myerror("FATType not expected (%u)!", fs->FATType);
+	  return NULL;
+	}
+	
 	if (device_seekset(fs->device, BSOffset + nr * FATSizeInBytes) == -1) {
 		myerror("Seek error!");
 		free(FAT);
 		return NULL;
 	}
-	if (device_read(fs->device, FAT, FATSizeInBytes, 1) < 1) {
+	if (device_read(fs->device, FAT, FATSizeInBytes, 1) < FATSizeInBytes) {
 		myerror("Failed to read from file!");
 		free(FAT);
 		return NULL;
@@ -568,7 +582,7 @@ int32_t checkFATs(struct sFileSystem *fs) {
 
 	off_t BSOffset;
 
-	char *FAT1, *FATx;
+	char *FATx;
 
 	// if there is just one FAT, we don't have to check anything
 	if (fs->FATCount < 2) {
@@ -577,51 +591,42 @@ int32_t checkFATs(struct sFileSystem *fs) {
 
 	FATSizeInBytes = fs->FATSize * fs->sectorSize;
 
-	if ((FAT1=malloc(FATSizeInBytes))==NULL) {
-		stderror();
-		return -1;
-	}
 	if ((FATx=malloc(FATSizeInBytes))==NULL) {
 		stderror();
-		free(FAT1);
 		return -1;
 	}
 	BSOffset = (off_t)SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
 	if (device_seekset(fs->device, BSOffset) == -1) {
 		myerror("Seek error!");
-		free(FAT1);
 		free(FATx);
 		return -1;
 	}
-	if (device_read(fs->device, FAT1, FATSizeInBytes, 1) < 1) {
+	if (device_read(fs->device, fs->FAT, FATSizeInBytes, 1) < 1) {
 		myerror("Failed to read from file!");
-		free(FAT1);
 		free(FATx);
 		return -1;
 	}
 
 	for(i=1; i < fs->FATCount; i++) {
-		if (device_seekset(fs->device, BSOffset+FATSizeInBytes) == -1) {
+		if (device_seekset(fs->device, BSOffset+i*FATSizeInBytes) == -1) {
 			myerror("Seek error!");
-			free(FAT1);
 			free(FATx);
 			return -1;
 		}
 		if (device_read(fs->device, FATx, FATSizeInBytes, 1) < 1) {
 			myerror("Failed to read from file!");
-			free(FAT1);
+
 			free(FATx);
 			return -1;
 		}
 
 		//printf("FAT1: %08lx FATx: %08lx\n", FAT1[0], FATx[0]);
 
-		result = memcmp(FAT1, FATx, FATSizeInBytes) != 0;
+		result = memcmp(fs->FAT, FATx, FATSizeInBytes) != 0;
 		if (result) break; // FATs don't match
 
 	}
 
-	free(FAT1);
 	free(FATx);
 
 	return result;
@@ -635,69 +640,35 @@ int32_t getFATEntry(struct sFileSystem *fs, uint32_t cluster, uint32_t *data) {
 	assert(fs != NULL);
 	assert(data != NULL);
 
-	off_t FATOffset, BSOffset;
-
 	*data=0;
-
-	switch(fs->FATType) {
+	
+  switch(fs->FATType) {
 	case FATTYPE_FAT32:
-		FATOffset = (off_t)cluster * 4;
-		BSOffset = (off_t)SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec) + FATOffset;
-		if (device_seekset(fs->device, BSOffset) == -1) {
-			myerror("Seek error!");
-			return -1;
-		}
-		if (device_read(fs->device, data, 4, 1) < 1) {
-			myerror("Failed to read from file!");
-			return -1;
-		}
-		*data=SwapInt32(*data);
-		*data = *data & 0x0fffffff;
+
+		*data=SwapInt32(((uint32_t *) fs->FAT)[cluster]) & 0x0fffffff;
+
 		break;
 	case FATTYPE_FAT16:
-		FATOffset = (off_t)cluster * 2;
-		BSOffset = (off_t) SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec) + FATOffset;
-		if (device_seekset(fs->device, BSOffset) == -1) {
-			myerror("Seek error!");
-			return -1;
-		}
-		if (device_read(fs->device, data, 2, 1)<1) {
-			myerror("Failed to read from file!");
-			return -1;
-		}
-		*data=SwapInt32(*data);
+		*data=((uint16_t *) fs->FAT)[cluster];
+		*data=SwapInt16(*data);
 		break;
 	case FATTYPE_FAT12:
-		FATOffset = (off_t) cluster + (cluster / 2);
-		BSOffset = (off_t) SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RsvdSecCnt) * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec) + FATOffset;
-		if (device_seekset(fs->device, BSOffset) == -1) {
-			myerror("Seek error!");
-			return -1;
-		}
-		if (device_read(fs->device, data, 2, 1)<1) {
-			myerror("Failed to read from file!");
-			return -1;
-		}
-
-		*data=SwapInt32(*data);
-
+	
+	 
+		*data=*((uint16_t *) (fs->FAT + cluster + (cluster/2)));
+		//fprintf(stderr, "data=%p\n", fs->FAT + cluster + (cluster/2));
+	  //fprintf(stderr, "*data=%04x\n", *data);
+		*data=SwapInt16(*data);
+		
 		if (cluster & 1)  {
-			*data = *data >> 4;	/* cluster number is odd */
+			*data = *data >> 4;	// cluster number is odd
 		} else {
-			*data = *data & 0x0FFF;	/* cluster number is even */
+			*data = *data & 0x0FFF;	// cluster number is even
 		}
+    //fprintf(stderr, "*data=%04x\n", *data);
 		break;
 	case FATTYPE_EXFAT:
-		FATOffset = (off_t)cluster * 4;
-		BSOffset = (off_t)SwapInt32(fs->bs.xxFATxx.exFAT.fat_sector_start) * fs->sectorSize + FATOffset;
-		if (device_seekset(fs->device, BSOffset) == -1) {
-			myerror("Seek error!");
-			return -1;
-		}
-		if (device_read(fs->device, data, 4, 1) < 1) {
-			myerror("Failed to read from file!");
-			return -1;
-		}
+		*data=((uint32_t *) fs->FAT)[cluster];
 		*data=SwapInt32(*data);
 		break;
 	default:
@@ -1136,6 +1107,9 @@ int32_t openFileSystem(char *path, uint32_t mode, struct sFileSystem *fs) {
 		fs->FATCount = fs->bs.xxFATxx.exFAT.fat_count;
 		fs->maxClusterChainLength = 0xffffffff; // basically not limited
 		fs->firstDataSector= SwapInt32(fs->bs.xxFATxx.exFAT.cluster_sector_start);
+		
+		// read FAT
+	  fs->FAT=readFAT(fs, 0);
 
 		if (checkVbrCecksum(fs)) {
 			myerror("Volume Boot Record checksum verification failed!");
@@ -1195,11 +1169,18 @@ int32_t openFileSystem(char *path, uint32_t mode, struct sFileSystem *fs) {
 			return -1;
 		}
 
+    /*fprintf(stderr, "BS_FATSz16: %04x\n", fs->bs.xxFATxx.FAT12_16_32.BS_FATSz16);
+    fprintf(stderr, "BS_FATSz32: %08x\n", fs->bs.xxFATxx.FAT12_16_32.FATxx.FAT32.BS_FATSz32);*/
 		if (fs->bs.xxFATxx.FAT12_16_32.BS_FATSz16 != 0) {
 			fs->FATSize = SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_FATSz16);
 		} else {
 			fs->FATSize = SwapInt32(fs->bs.xxFATxx.FAT12_16_32.FATxx.FAT32.BS_FATSz32);
 		}
+		
+		fs->sectorSize=SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
+		
+		// read FAT
+	  fs->FAT=readFAT(fs, 0);
 
 		// check whether count of root dir entries is ok for given FAT type
 		if (((fs->FATType == FATTYPE_FAT16) || (fs->FATType == FATTYPE_FAT12)) && (SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_RootEntCnt) == 0)) {
@@ -1224,8 +1205,6 @@ int32_t openFileSystem(char *path, uint32_t mode, struct sFileSystem *fs) {
 			return -1;
 		}
 
-		fs->sectorSize=SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
-
 		fs->clusterSize=fs->bs.xxFATxx.FAT12_16_32.BS_SecPerClus * SwapInt16(fs->bs.xxFATxx.FAT12_16_32.BS_BytesPerSec);
 
 		fs->FSSize = (uint64_t) fs->clusters * fs->clusterSize;
@@ -1244,7 +1223,6 @@ int32_t openFileSystem(char *path, uint32_t mode, struct sFileSystem *fs) {
 
 	fs->maxDirEntriesPerCluster = fs->clusterSize / DIR_ENTRY_SIZE;
 
-
 #ifndef __WIN32__
 	// convert utf 16 le to local charset
 	fs->cd = iconv_open("", "UTF-16LE");
@@ -1274,6 +1252,8 @@ int32_t closeFileSystem(struct sFileSystem *fs) {
 	closes file system
 */
 	assert(fs != NULL);
+	
+	free(fs->FAT);
 
 	device_close(fs->device);
 #ifndef __WIN32__
