@@ -79,48 +79,273 @@ DEVICE *device_open(const char *path) {
   }
 
   dev->fd=fd;
+  dev->bufferedBlock=-1;
+  dev->unwrittenChanges=0;
+  dev->offset=0;
+  dev->deviceOffset=0;
 
   return dev;
 }
 
+int64_t readBlock(DEVICE *device, int64_t deviceOffset, void *data) {
+
+  assert(device != NULL);
+  assert(deviceOffset >= 0);
+  assert(data != NULL);
+
+  int64_t ret;  
+  
+  // seek only if device offset needs to be changed
+  if (deviceOffset  != device->deviceOffset) {
+    // seek to block   
+  #if defined __BSD__ || defined __OSX__ 
+    ret=lseek(device->fd, (off_t) deviceOffset * DEVICE_BLOCK_SIZE, SEEK_SET);
+  #else
+    ret=lseek64(device->fd, (off64_t) deviceOffset * DEVICE_BLOCK_SIZE, SEEK_SET);
+  #endif
+    if (ret==-1) {
+      myerror("Failed to seek to block (%u)!", deviceOffset);
+    }
+  }
+  
+  ret=read(device->fd, device->bufferedData, DEVICE_BLOCK_SIZE);
+  if (ret < DEVICE_BLOCK_SIZE) {
+    stderror();
+    myerror("Read failed (%u)!", ret);
+    return -1;
+  }
+  
+  device->deviceOffset=deviceOffset+1;
+  
+  return 0;
+    
+}
+
+int64_t writeBlock(DEVICE *device, int64_t deviceOffset, void *data) {
+
+  assert(device != NULL);
+  assert(deviceOffset >= 0);
+  assert(data != NULL);
+  
+  int64_t ret;  
+
+  // seek only if device offset needs to be changed
+  if (deviceOffset != device->deviceOffset) {
+    // seek to block   
+  #if defined __BSD__ || defined __OSX__ 
+    ret=lseek(device->fd, (off_t) deviceOffset * DEVICE_BLOCK_SIZE, SEEK_SET);
+  #else
+    ret=lseek64(device->fd, (off64_t) deviceOffset * DEVICE_BLOCK_SIZE, SEEK_SET);
+  #endif
+    if (ret==-1) {
+      myerror("Failed to seek to block (%u)!", deviceOffset);
+    }
+  }
+  
+  ret=write(device->fd, device->bufferedData, DEVICE_BLOCK_SIZE);
+  if (ret < DEVICE_BLOCK_SIZE) {
+    stderror();
+    myerror("Write failed (%u)!", ret);
+    return -1;
+  }
+  
+  device->deviceOffset=deviceOffset+1;
+  
+  return 0;
+    
+}
+
+int64_t flushBuffer(DEVICE *device) {
+  assert(device != NULL);
+
+  if ((device->bufferedBlock!= -1) && device->unwrittenChanges) {
+  
+    if (writeBlock(device, device->bufferedBlock, device->bufferedData) !=0) {
+      myerror("Failed to write block!");
+      return -1;
+    }
+
+    device->unwrittenChanges=0;
+    
+    return 1; // no flush
+  }
+  
+  return 0; // flush 
+}
+
 int64_t device_seekset(DEVICE *device, int64_t offset) {
 
   assert(device != NULL);
   assert(offset >= 0);
+  
+  // if block changes, flush buffered block
+  if ((device->offset / DEVICE_BLOCK_SIZE) != (offset / DEVICE_BLOCK_SIZE)) {
+    flushBuffer(device);
+    device->bufferedBlock=-1;
+  }
+  
+  device->offset=offset;
+  
+  return 0;
 
-#if defined __BSD__ || defined __OSX__ 
-  return lseek(device->fd, (off_t) offset, SEEK_SET);
-#else
-  return lseek64(device->fd, (off64_t) offset, SEEK_SET);
-#endif
 }
 
 int64_t device_read(DEVICE *device, void *data, uint64_t size, uint64_t n) {
 
   assert(device != NULL);
   assert(data != NULL);
+  assert(size * n > 0);
+  
+  uint64_t bytesLeft=size*n;
+  int64_t copyBytes;
+  uint64_t bytesLeftInBlock;
+  int64_t currentBlock;
+ 
+  while(bytesLeft) { // loop while blocks need to be read
+  
+    currentBlock = device->offset / DEVICE_BLOCK_SIZE;
+     
+    // if block is not buffered
+    if (device->bufferedBlock != currentBlock) { // read new block
+        
+      // make sure that all unwritten block data is written before buffering new block
+      flushBuffer(device);
+      
+      // buffer new block
+      if (readBlock(device, currentBlock, device->bufferedData) != 0) {
+        myerror("Failed to read block!");
+        return -1;
+      }
+      
+      // update buffered block number
+      device->bufferedBlock=currentBlock;
+ //   } else {
+ //     fprintf(stderr, "Block %08lx already buffered!\n", currentBlock);
+    }
+    
+    bytesLeftInBlock = DEVICE_BLOCK_SIZE - device->offset % DEVICE_BLOCK_SIZE;
+     
+    if (bytesLeft <= bytesLeftInBlock) {
+      copyBytes=bytesLeft;
+    } else {
+      copyBytes=bytesLeftInBlock;
+    }
 
-  return read(device->fd, data, (size_t) size * n);
+    // copy data from current block
+    memcpy(data, device->bufferedData + device->offset % DEVICE_BLOCK_SIZE, copyBytes); 
+
+    // update data pointer
+    data+=copyBytes;
+
+    // update device offset after read
+    device->offset+=copyBytes;
+
+    // decrease number of bytes to be copied
+    bytesLeft -= copyBytes;
+        
+  }
+    
+  return size*n;
+  
 }
 
 int64_t device_write(DEVICE *device, const void *data, uint64_t size, uint64_t n) {
 
   assert(device != NULL);
   assert(data != NULL);
+  assert(size * n > 0);
+  
+  uint64_t bytesLeft=size*n;
+  int64_t copyBytes;
+ 
+  while(bytesLeft) { // loop while blocks need to be written
+    
+    // next block will be written til end
+    if (bytesLeft >= (uint64_t) (DEVICE_BLOCK_SIZE - device->offset % DEVICE_BLOCK_SIZE)) {
+  
+       // if block not buffered and we still need some bytes, buffer it first
+      if ((device->offset % DEVICE_BLOCK_SIZE !=0) &&
+          (device->offset / DEVICE_BLOCK_SIZE != device->bufferedBlock)) {
+      
+        // flush if some other block is buffered
+        flushBuffer(device);
+        
+        // read block
+        if (readBlock(device, device->offset / DEVICE_BLOCK_SIZE, device->bufferedData) != 0) {
+          myerror("Failed to read block!");
+          return -1;
+        }
+
+      }
+    
+      copyBytes=DEVICE_BLOCK_SIZE - device->offset % DEVICE_BLOCK_SIZE;
+   
+      // copy data to buffer
+      memcpy(device->bufferedData+device->offset % DEVICE_BLOCK_SIZE, data, copyBytes);
+      
+      // block is now buffered even if it wasn't before
+      device->bufferedBlock=device->offset / DEVICE_BLOCK_SIZE;
+     
+      // flush buffer 
+      device->unwrittenChanges=1;
+      flushBuffer(device);
+      
+    } else { // block will not be written completely (last block)
+    
+       // if block not buffered, buffer it first
+      if (device->offset / DEVICE_BLOCK_SIZE != device->bufferedBlock) {
+      
+        // flush if some other block is buffered
+        flushBuffer(device);
+        
+        if (readBlock(device, device->offset / DEVICE_BLOCK_SIZE, device->bufferedData) != 0) {
+          myerror("Failed to read block!");
+          return -1;
+        }
+        
+        // update buffered block number
+        device->bufferedBlock=device->offset / DEVICE_BLOCK_SIZE;
+        
+      }
+    
+      copyBytes=bytesLeft;
+    
+      // copy data to buffer
+      memcpy(device->bufferedData+device->offset % DEVICE_BLOCK_SIZE, data, copyBytes);      
+
+      // we will not write yet as block is not full 
+      device->unwrittenChanges=1;  
+
+    }
+    
+    // increase data pointer
+    data+=copyBytes;
+    
+    // increase device offset
+    device->offset+=copyBytes;
+    
+    // decrease remaining bytes
+    bytesLeft-=copyBytes;  
+  }
+  
+  return size*n;
 
-  return write(device->fd, (void *) data, (size_t) size * n);
 }
 
 int device_sync(DEVICE *device) {
 
   assert(device != NULL);
 
+  flushBuffer(device);
+
   return fsync(device->fd);
 }
 
 int device_close(DEVICE *device) {
 
   assert(device != NULL);
+ 
+  flushBuffer(device);
 
   if(!close(device->fd)) {
     free((void*) device);
