@@ -31,8 +31,14 @@
 
 #define DIRECTORY_SEPARATOR '/'
 
+#define DEVICE_BLOCK_SIZE 512
+
 typedef struct {
   int fd;
+  int64_t bufferedBlock;
+  int64_t offset, deviceOffset;
+  char bufferedData[DEVICE_BLOCK_SIZE];
+  int unwrittenChanges;
 } DEVICE;
 
 #elif defined __WIN32__
@@ -47,7 +53,7 @@ typedef struct {
   HANDLE h;
   int isDrive;
   uint16_t sectorSize;
-  uint64_t currentOffset;
+  uint64_t deviceOffset;
   char *buffer;
   uint64_t bufferedSector;
   int unwrittenChanges;
