@@ -174,10 +174,10 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 
 	char s1[MAX_PATH_LEN+1];
 	char s2[MAX_PATH_LEN+1];
-	char s1_col[MAX_PATH_LEN*2+1];
-	char s2_col[MAX_PATH_LEN*2+1];
 	char scase1[MAX_PATH_LEN+1];
 	char scase2[MAX_PATH_LEN+1];
+	char s1_col[MAX_PATH_LEN*4+1];
+	char s2_col[MAX_PATH_LEN*4+1];
 
 	uint16_t i;
 
@@ -269,11 +269,13 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
 			i++;
 		}
 		ss1=scase1;
+		scase1[i]='\0';
 		i=0;
 		while(ss2[i]) {
 			scase2[i] = tolower(ss2[i]);
 			i++;
 		}
+		scase2[i]='\0';
 		ss2=scase2;
 	}
 
@@ -285,8 +287,9 @@ int32_t cmpEntries(struct sDirEntryList *de1, struct sDirEntryList *de2) {
  	} else {
 
 		// consider locale for comparison
-		if ((strxfrm(s1_col, ss1, MAX_PATH_LEN*2) == MAX_PATH_LEN*2) ||
-		    (strxfrm(s2_col, ss2, MAX_PATH_LEN*2) == MAX_PATH_LEN*2)) {
+		// Note: strxfrm with strcmp ist faster than strcoll!
+		if ((strxfrm(s1_col, ss1, MAX_PATH_LEN*4) >= MAX_PATH_LEN*4) ||
+		    (strxfrm(s2_col, ss2, MAX_PATH_LEN*4) >= MAX_PATH_LEN*4)) {
 			myerror("String collation error!");
 			exit(1);
 		}
@@ -303,6 +306,7 @@ void insertDirEntryList(struct sDirEntryList *new, struct sDirEntryList *list, u
 
 	assert(new != NULL);
 	assert(list != NULL);
+	assert(reordered != NULL);
 
 	struct sDirEntryList *tmp, *dummy;
 
@@ -314,7 +318,7 @@ void insertDirEntryList(struct sDirEntryList *new, struct sDirEntryList *list, u
 	}
 
 	// entry needed to be reordered
-	*reordered = (tmp->next != NULL);
+	*reordered = (tmp->next != NULL) ? 1 : 0;
 
 	dummy=tmp->next;
 	tmp->next=new;
@@ -504,7 +508,7 @@ int32_t	insertExFATDirEntrySet(struct sExFATDirEntrySetList *desl, struct sExFAT
 */
 	assert(desl != NULL);
 	assert(new != NULL);
-  assert(reordered != NULL);
+ 	assert(reordered != NULL);
 
 	struct sExFATDirEntrySetList *tmp, *tmpl, *dummy;
 
@@ -520,8 +524,8 @@ int32_t	insertExFATDirEntrySet(struct sExFATDirEntrySetList *desl, struct sExFAT
 		tmp=tmp->next;
 	}
 
-  // reordered?
-  *reordered = (tmp->next != NULL) ? 1 : 0;
+  	// reordered?
+  	*reordered = (tmp->next != NULL) ? 1 : 0;
 
 	dummy=tmp->next;
 	tmp->next=tmpl;
@@ -591,11 +595,11 @@ int32_t cmpExFATDirEntrySets(struct sExFATDirEntrySet *des1, struct sExFATDirEnt
 
 	char s1[MAX_PATH_LEN+1];
 	char s2[MAX_PATH_LEN+1];
-	char s1_col[MAX_PATH_LEN*2+1];
-	char s2_col[MAX_PATH_LEN*2+1];
 	char scase1[MAX_PATH_LEN+1];
 	char scase2[MAX_PATH_LEN+1];
-
+	char s1_col[MAX_PATH_LEN*4+1];
+	char s2_col[MAX_PATH_LEN*4+1];
+	
 	uint16_t i;
 
 	// the volume label must always remain at the beginning of the (root) directory
@@ -680,12 +684,14 @@ int32_t cmpExFATDirEntrySets(struct sExFATDirEntrySet *des1, struct sExFATDirEnt
 			scase1[i] = tolower(ss1[i]);
 			i++;
 		}
+		scase1[i]='\0';
 		ss1=scase1;
 		i=0;
 		while(ss2[i]) {
 			scase2[i] = tolower(ss2[i]);
 			i++;
 		}
+		scase2[i]='\0';
 		ss2=scase2;
 	}
 
@@ -697,8 +703,9 @@ int32_t cmpExFATDirEntrySets(struct sExFATDirEntrySet *des1, struct sExFATDirEnt
  	} else {
 
 		// consider locale for comparison
-		if ((strxfrm(s1_col, ss1, MAX_PATH_LEN*2) == MAX_PATH_LEN*2) ||
-		    (strxfrm(s2_col, ss2, MAX_PATH_LEN*2) == MAX_PATH_LEN*2)) {
+		// Note: strxfrm with strcmp ist faster than strcoll!
+		if ((strxfrm(s1_col, ss1, MAX_PATH_LEN*4) >= MAX_PATH_LEN*4) ||
+		    (strxfrm(s2_col, ss2, MAX_PATH_LEN*4) >= MAX_PATH_LEN*4)) {
 			myerror("String collation error!");
 			exit(1);
 		}
