@@ -45,63 +45,64 @@
 
 // program information
 #define INFO_PROGRAM		"fatsort"
-#define INFO_VERSION		"1.6.5"
+#define INFO_VERSION		"1.7"
 #define INFO_AUTHOR		"Written by Boris Leidner.\n"
-#define INFO_COPYRIGHT		"Copyright (C) 2004-2023 Boris Leidner.\n"
+#define INFO_COPYRIGHT		"Copyright (C) 2004-2025 Boris Leidner.\n"
 #define INFO_LICENSE		"License GPLv2: GNU GPL version 2 (see LICENSE.txt)\n" \
-				"This is free software: you are free to change and redistribute it.\n" \
+				"This is free software: you are free to change and " \
+				"redistribute it.\n" \
 				"There is NO WARRANTY, to the extent permitted by law.\n"
-#define INFO_DESCRIPTION	"FATSort sorts directory structures of FAT file systems (FAT12, FAT16, FAT32 and exFAT). " \
-				"Many MP3 hardware players don't sort files automatically " \
-				"but play them in the  order they were transferred to the " \
-				"device. FATSort can help here.\n"
+#define INFO_DESCRIPTION	"FATSort sorts the directory structure of FAT file " \
+				"systems, including FAT12,\nFAT16, FAT32, and exFAT.\n" \
 
 #define INFO_USAGE		"Usage: fatsort [OPTIONS] DEVICE\n" \
 				"\n" \
 				"Sort options:\n\n" \
-				"\t-a\tUse ASCIIbetical order for sorting\n\n" \
-				"\t-c\tIgnore case of file names (default for most locales)\n\n" \
-				"\t-I PFX\tIgnore file name PFX\n\n" \
-				"\t-n\tNatural order sorting\n\n" \
-				"\t-o FLAG\tSort order of files where FLAG is one of\n\n" \
+				"\t-a\tUse ASCII order for sorting.\n\n" \
+				"\t-c\tIgnore case of file names (default for most locales).\n\n" \
+				"\t-I PFX\tIgnore the prefix PFX when comparing file names.\n\n" \
+				"\t-n\tUse natural order sorting.\n\n" \
+				"\t-o FLAG\tSet the sort order of files, where FLAG is one of:\n\n" \
 				"\t\t\td : directories first (default)\n\n" \
 				"\t\t\tf : files first\n\n" \
-				"\t\t\ta : files and directories are not differentiated\n\n" \
-				"\t-r\tSort in reverse order\n\n" \
-				"\t-R\tSort in random order\n\n" \
-				"\t\tThis options makes all other sort options obsolete.\n\n" \
-				"\t-t\tSort by last modification date and time\n\n" \
-				"The following options can be specified multiple times\n" \
-				"to select which directories shall be sorted:\n\n" \
-				"\t-d DIR\tSort directory DIR only\n\n" \
-				"\t-D DIR\tSort directory DIR and all subdirectories\n\n" \
-				"\t-x DIR\tDon't sort directory DIR\n\n" \
-				"\t-X DIR\tDon't sort directory DIR and its subdirectories\n\n" \
-				"The following options can be specified multiple times\n" \
-				"to select which directories shall be sorted using\n" \
-				"POSIX.2 extended regular expressions:\n\n" \
-				"\t-e RE\tOnly sort directories that match regular expression RE\n\n" \
-				"\t-E RE\tDon't sort directories that match regular expression RE\n\n" \
-				"However, options -e and -E may not be used simultaneously with\n" \
-				"options -d, -D, -x and -X.\n\n" \
-				"\t-L LOC\tUse the locale LOC instead of the locale from the environment variables\n\n" \
+				"\t\t\ta : do not distinguish between files and directories\n\n" \
+				"\t-r\tSort in reverse order.\n\n" \
+				"\t-R\tSort in random order.\n\n" \
+				"\t\tThis option overrides all other sort options.\n\n" \
+				"\t-t\tSort by last modification date and time.\n\n" \
+				"The following options can be specified multiple times to control\n" \
+				"which directories are sorted:\n\n" \
+				"\t-d DIR\tSort the specified directory DIR only.\n\n" \
+				"\t-D DIR\tSort DIR and all its subdirectories.\n\n" \
+				"\t-x DIR\tExclude DIR from sorting.\n\n" \
+				"\t-X DIR\tExclude DIR and all its subdirectories from sorting.\n\n" \
+				"The following options can be specified multiple times to select\n" \
+				"directories using POSIX.2 extended regular expressions:\n\n" \
+				"\t-e RE\tOnly sort directories matching regular expression RE.\n\n" \
+				"\t-E RE\tDon't sort directories matching regular expression RE.\n\n" \
+				"Note: Options -e and -E cannot be used together with -d, -D, -x and -X.\n\n" \
+				"\t-L LOC\tUse the specified locale LOC instead of the system locale.\n" \
+				"\t\tFATSort follows locale-specific rules when sorting file names\n" \
+				"\t\tand directories.\n\n" \
 				"More options:\n\n" \
-				"\t-l\tPrint current order of files only\n\n" \
-				"\t-i\tPrint file system information only\n\n" \
-				"\t-f\tForce sorting even if file system is mounted\n\n" \
+				"\t-l\tList the current file order only.\n\n" \
+				"\t-i\tDisplay file system information only.\n\n" \
+				"\t-f\tForce sorting even if the file system is mounted.\n\n" \
 				"\t-h, --help\n\n" \
-				"\t\tPrint some help\n\n" \
+				"\t\tDisplay this help message.\n\n" \
 				"\t-v, --version\n\n" \
-				"\t\tPrint version information\n\n" \
-				"\t-q\tBe quiet\n\n" \
+				"\t\tShow version information.\n\n" \
+				"\t-q\tSuppress all non-error output.\n\n" \
 				"DEVICE must be a FAT12, FAT16, FAT32 or exFAT file system.\n\n" \
-				"WARNING: THE FILESYSTEM MUST BE CONSISTENT (NO FILESYSTEM ERRORS).\n" \
-				"PLEASE BACKUP YOUR DATA BEFORE USING FATSORT. RISK OF CORRUPT FILESYSTEM!\n" \
-				"FATSORT USER ASSUMES ALL RISK. FATSORT WILL NOT BE HELD LIABLE FOR DATA LOSS!\n" \
-								"\n" \
+				"WARNING:\n" \
+				"THE FILE SYSTEM MUST BE CONSISTENT (I.E., FREE OF ERRORS) BEFORE RUNNING \n" \
+				"FATSORT. THERE IS A RISK OF FILE SYSTEM CORRUPTION. " \
+				"ALWAYS BACK UP YOUR DATA.\n\n" \
+				"Environment:\n" \
+				"The environment variable LC_ALL determines the system locale used by FATSort.\n\n" \
 				"Examples:\n" \
 				"\tfatsort /dev/sda\t\tSort /dev/sda.\n" \
-				"\tfatsort -n /dev/sdb1\t\tSort /dev/sdb1 with natural order.\n" \
+				"\tfatsort -n /dev/sdb1\t\tSort /dev/sdb1 using natural order.\n" \
 				"\n" \
 				"Report bugs to <fatsort@formenos.de>.\n"
 
